@@ -52,15 +52,18 @@
     hist(x$s2[burnin:x$T], ...)
     plot(burnin:x$T, x$s2[burnin:x$T], type="l", main="s2 chain", ...)
   } else if(which == "lambda2") {
+    if(is.null(x$lambda2)) stop("LASSO regression was not used")
     par(mfrow=c(2,1))
     hist(x$lambda2[burnin:x$T], ...)
     plot(burnin:x$T, x$lambda2[burnin:x$T], type="l", main="lambda2 chain", ...)
   } else if(which == "m") {
+    if(!x$RJ) stop("RJ was not used")
     par(mfrow=c(2,1))
     hist(x$m[burnin:x$T], ...)
     plot(burnin:x$T, x$m[burnin:x$T], type="l", main="m chain", ...)
   }
   else if(which == "tau2i"){
+    if(is.null(x$tau2i)) stop("LASSO regression was not used")
     boxplot(data.frame(x$tau2i[burnin:x$T,]),
             main="Boxplot of tau2i", ylab="tau2i", ...)
   }
@@ -81,6 +84,7 @@ function(object, burnin=0, ...)
     ## check the burnin argument
     if(length(burnin) != 1 || burnin<0 || burnin >= object$T)
       stop("burnin must be a non-negative scalar < object$T")
+    burnin <- burnin+1
 
     ## make the list
     rl <- list(call=object$call, B=burnin, T=object$T, thin=object$thin)
@@ -93,14 +97,14 @@ function(object, burnin=0, ...)
     rl$s2 <- summary(object$s2[burnin:object$T])
 
     ## only do if lasso
-    if(object$lambda2[1] != 0) {
+    if(!is.null(object$lambda2))
       rl$lambda2 <- summary(object$lambda2[burnin:object$T])
+    if(!is.null(object$tau2i))
       rl$tau2i <- summary(data.frame(object$tau2i[burnin:object$T,]))
-    }
 
     ## only do if RJ
     rl$bn0 <- apply(object$beta[burnin:object$T,], 2,
-                   function(x){ sum(x != 0) })/(object$T-burnin)
+                   function(x){ sum(x != 0) })/(object$T-burnin+1)
     
     ## print it or return it
     rl
@@ -176,8 +180,11 @@ function(x, ...)
     cat("each sample\n")
 
     ## say something about lasso and RJ
-    if(x$lambda2[1] != 0)
+    if(!is.null(x$tau2i))
       cat("\nLasso was used to shrink regression coefficients\n")
+    else if(!is.null(x$lambda2)) {
+      cat("\nA ridge parameter was used to shrink regression coefficients\n")
+    }
     if(x$RJ) {
       cat("\nReversible Jump (RJ) was used to average over subsets\n")
       cat("of columns in the design matrix, allowing a maximum of", x$M, "\n")
